package kz.project.printedFormsService.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import kz.project.printedFormsService.config.SecurityContextUtils;
import kz.project.printedFormsService.data.dto.DocumentByDateDto;
import kz.project.printedFormsService.data.dto.DocumentByTemplateDto;
import kz.project.printedFormsService.exception.ValidationException;
import kz.project.printedFormsService.service.DocumentsService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;

@RestController
@RequiredArgsConstructor
@Slf4j
@Tag(name = "Documents Controller", description = "API DocumentsService")
public class DocumentsController {

    private final DocumentsService documentsService;

    public static final String BASE_PATH = "/api/documents";
    public static final String BY_DAY = BASE_PATH + "/by-day";
    public static final String BY_TEMPLATE = BASE_PATH + "/by-template";

    @GetMapping(BY_DAY)
    @Operation(description = "Метод для получения данных по общему количеству генерации по всем доступным пользователю шаблонам")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = DocumentByDateDto.class))
                    }),


            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка валидации параметров запроса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })

    public ResponseEntity<DocumentByDateDto> getDocumentsCountByDay(
            @RequestParam("startDate")
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate startDate,
            @RequestParam("endDate")
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate endDate) {
        Map<String, List<String>> projectRoleMap = SecurityContextUtils.getProjectRoleMap();
        DocumentByDateDto documentsByDate = documentsService.getDocumentCountsByDate(startDate, endDate, projectRoleMap.keySet());
        return ResponseEntity.ok(documentsByDate);
    }



    @GetMapping(BY_TEMPLATE)
    @Operation(description = "Метод для получения данных по общему количеству генерации по каждому доступному пользователю шаблонам")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = DocumentByTemplateDto.class))
                    }),


            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка валидации параметров запроса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })

    public ResponseEntity<DocumentByTemplateDto> getDocumentsCountByTemplate(
            @RequestParam("startDate")
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate startDate,
            @RequestParam("endDate")
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate endDate) {
        Map<String, List<String>> projectRoleMap = SecurityContextUtils.getProjectRoleMap();
        DocumentByTemplateDto documentCountsByTemplate = documentsService.getDocumentCountsByTemplate(startDate, endDate, projectRoleMap.keySet());
        return ResponseEntity.ok(documentCountsByTemplate);
    }
}
